@@ -68,7 +68,7 @@ public:
     /// use ModuleList::GetSharedModule().
     ///
     /// @param[in] file_spec
-    ///     The file specification for the on disk repesentation of
+    ///     The file specification for the on disk representation of
     ///     this executable image.
     ///
     /// @param[in] arch
@@ -88,10 +88,14 @@ public:
     Module (const FileSpec& file_spec,
             const ArchSpec& arch,
             const ConstString *object_name = NULL,
-            off_t object_offset = 0,
+            lldb::offset_t object_offset = 0,
             const TimeValue *object_mod_time_ptr = NULL);
 
     Module (const ModuleSpec &module_spec);
+    
+    static lldb::ModuleSP
+    CreateJITModule (const lldb::ObjectFileJITDelegateSP &delegate_sp);
+    
     //------------------------------------------------------------------
     /// Destructor.
     //------------------------------------------------------------------
@@ -113,10 +117,18 @@ public:
     /// @param[in] target
     ///     The target in which to apply the section load addresses.
     ///
-    /// @param[in] offset
-    ///     The offset to apply to all file addresses for all top 
-    ///     level sections in the object file as each section load
-    ///     address is being set.
+    /// @param[in] value
+    ///     if \a value_is_offset is true, then value is the offset to
+    ///     apply to all file addresses for all top level sections in
+    ///     the object file as each section load address is being set.
+    ///     If \a value_is_offset is false, then "value" is the new
+    ///     absolute base address for the image.
+    ///
+    /// @param[in] value_is_offset
+    ///     If \b true, then \a value is an offset to apply to each
+    ///     file address of each top level section.
+    ///     If \b false, then \a value is the image base address that
+    ///     will be used to rigidly slide all loadable sections.
     ///
     /// @param[out] changed
     ///     If any section load addresses were changed in \a target,
@@ -133,7 +145,8 @@ public:
     //------------------------------------------------------------------
     bool
     SetLoadAddress (Target &target, 
-                    lldb::addr_t offset, 
+                    lldb::addr_t value,
+                    bool value_is_offset,
                     bool &changed);
     
     //------------------------------------------------------------------
@@ -184,7 +197,7 @@ public:
     /// in a module.
     ///
     /// @param[in] s
-    ///     The stream to which to dump the object descripton.
+    ///     The stream to which to dump the object description.
     //------------------------------------------------------------------
     void
     Dump (Stream *s);
@@ -406,7 +419,7 @@ public:
                          VariableList& variable_list);
 
     //------------------------------------------------------------------
-    /// Find global and static variables by regular exression.
+    /// Find global and static variables by regular expression.
     ///
     /// @param[in] regex
     ///     A regular expression to use when matching the name.
@@ -459,11 +472,11 @@ public:
     ///
     /// @param[in] type_name
     ///     The name of the type we are looking for that is a fully
-    ///     or partially qualfieid type name.
+    ///     or partially qualified type name.
     ///
     /// @param[in] exact_match
-    ///     If \b true, \a type_name is fully qualifed and must match
-    ///     exactly. If \b false, \a type_name is a partially qualfied
+    ///     If \b true, \a type_name is fully qualified and must match
+    ///     exactly. If \b false, \a type_name is a partially qualified
     ///     name where the leading namespaces or classes can be
     ///     omitted to make finding types that a user may type
     ///     easier.
@@ -569,6 +582,18 @@ public:
         m_platform_file = file;
     }
 
+    const FileSpec &
+    GetRemoteInstallFileSpec () const
+    {
+        return m_remote_install_file;
+    }
+    
+    void
+    SetRemoteInstallFileSpec (const FileSpec &file)
+    {
+        m_remote_install_file = file;
+    }
+    
     const FileSpec &
     GetSymbolFileFileSpec () const
     {
@@ -682,11 +707,25 @@ public:
     uint32_t
     GetVersion (uint32_t *versions, uint32_t num_versions);
 
-    // Load an object file from memory.
+    //------------------------------------------------------------------
+    /// Load an object file from memory. 
+    ///
+    /// If available, the size of the object file in memory may be 
+    /// passed to avoid additional round trips to process memory. 
+    /// If the size is not provided, a default value is used. This
+    /// value should be large enough to enable the ObjectFile plugins
+    /// to read the header of the object file without going back to the
+    /// process. 
+    ///
+    /// @return 
+    ///     The object file loaded from memory or NULL, if the operation 
+    ///     failed (see the `error` for more information in that case).
+    //------------------------------------------------------------------
     ObjectFile *
     GetMemoryObjectFile (const lldb::ProcessSP &process_sp, 
                          lldb::addr_t header_addr,
-                         Error &error);
+                         Error &error,
+                         size_t size_to_read = 512);
     //------------------------------------------------------------------
     /// Get the symbol vendor interface for the current architecture.
     ///
@@ -734,12 +773,12 @@ public:
     /// A debugging function that will cause everything in a module to
     /// be parsed.
     ///
-    /// All compile units will be pasred, along with all globals and
+    /// All compile units will be parsed, along with all globals and
     /// static variables and all functions for those compile units.
     /// All types, scopes, local variables, static variables, global
     /// variables, and line tables will be parsed. This can be used
     /// prior to dumping a module to see a complete list of the
-    /// resuling debug information that gets parsed, or as a debug
+    /// resulting debug information that gets parsed, or as a debug
     /// function to ensure that the module can consume all of the
     /// debug data the symbol vendor provides.
     //------------------------------------------------------------------
@@ -927,7 +966,7 @@ public:
 
     //------------------------------------------------------------------
     // Return true if the file backing this module has changed since the
-    // module was originally created  since we saved the intial file
+    // module was originally created  since we saved the initial file
     // modification time when the module first gets created.
     //------------------------------------------------------------------
     bool
@@ -1059,6 +1098,7 @@ protected:
     lldb_private::UUID          m_uuid;         ///< Each module is assumed to have a unique identifier to help match it up to debug symbols.
     FileSpec                    m_file;         ///< The file representation on disk for this module (if there is one).
     FileSpec                    m_platform_file;///< The path to the module on the platform on which it is being debugged
+    FileSpec                    m_remote_install_file;  ///< If set when debugging on remote platforms, this module will be installed at this location
     FileSpec                    m_symfile_spec; ///< If this path is valid, then this is the file that _will_ be used as the symbol file for this module
     ConstString                 m_object_name;  ///< The name an object within this module that is selected, or empty of the module is represented by \a m_file.
     uint64_t                    m_object_offset;
@@ -1134,7 +1174,9 @@ protected:
     friend class SymbolFile;
 
 private:
-
+    
+    Module (); // Only used internally by CreateJITModule ()
+    
     size_t
     FindTypes_Impl (const SymbolContext& sc, 
                     const ConstString &name,
