@@ -35,7 +35,7 @@ namespace lldb_private {
 /// ValueObject:
 ///
 /// This abstract class provides an interface to a particular value, be it a register, a local or global variable,
-/// that is evaluated in some particular scope.  The ValueObject also has the capibility of being the "child" of
+/// that is evaluated in some particular scope.  The ValueObject also has the capability of being the "child" of
 /// some other variable object, and in turn of having children.  
 /// If a ValueObject is a root variable object - having no parent - then it must be constructed with respect to some
 /// particular ExecutionContextScope.  If it is a child, it inherits the ExecutionContextScope from its parent.
@@ -380,7 +380,7 @@ public:
     GetTypeImpl ();
 
     //------------------------------------------------------------------
-    // Sublasses must implement the functions below.
+    // Subclasses must implement the functions below.
     //------------------------------------------------------------------
     virtual uint64_t
     GetByteSize() = 0;
@@ -389,11 +389,14 @@ public:
     GetValueType() const = 0;
 
     //------------------------------------------------------------------
-    // Sublasses can implement the functions below.
+    // Subclasses can implement the functions below.
     //------------------------------------------------------------------
     virtual ConstString
     GetTypeName();
     
+    virtual ConstString
+    GetDisplayTypeName();
+    
     virtual ConstString
     GetQualifiedTypeName();
 
@@ -465,7 +468,7 @@ public:
         return true;
     }
 
-    virtual off_t
+    virtual lldb::offset_t
     GetByteOffset()
     {
         return 0;
@@ -499,6 +502,10 @@ public:
     GetValueAsCString ();
     
     virtual bool
+    GetValueAsCString (const lldb_private::TypeFormatImpl& format,
+                       std::string& destination);
+
+    bool
     GetValueAsCString (lldb::Format format,
                        std::string& destination);
     
@@ -524,7 +531,7 @@ public:
     GetDeclaration (Declaration &decl);
 
     //------------------------------------------------------------------
-    // The functions below should NOT be modified by sublasses
+    // The functions below should NOT be modified by subclasses
     //------------------------------------------------------------------
     const Error &
     GetError();
@@ -615,7 +622,8 @@ public:
     DumpPrintableRepresentation (Stream& s,
                                  ValueObjectRepresentationStyle val_obj_display = eValueObjectRepresentationStyleSummary,
                                  lldb::Format custom_format = lldb::eFormatInvalid,
-                                 PrintableRepresentationSpecialCases special = ePrintableRepresentationSpecialCasesAllow);
+                                 PrintableRepresentationSpecialCases special = ePrintableRepresentationSpecialCasesAllow,
+                                 bool do_dump_error = true);
     bool
     GetValueIsValid () const;
 
@@ -707,6 +715,10 @@ public:
     {
     }
 
+    // Find the address of the C++ vtable pointer
+    virtual lldb::addr_t
+    GetCPPVTableAddress(AddressType &address_type);
+    
     virtual lldb::ValueObjectSP
     Cast (const ClangASTType &clang_ast_type);
     
@@ -757,7 +769,7 @@ public:
     
     static lldb::ValueObjectSP
     CreateValueObjectFromData (const char* name,
-                               DataExtractor& data,
+                               const DataExtractor& data,
                                const ExecutionContext& exe_ctx,
                                ClangASTType type);
     
@@ -788,7 +800,7 @@ public:
 					uint32_t item_count = 1);
     
     virtual uint64_t
-    GetData (DataExtractor& data);
+    GetData (DataExtractor& data, Error &error);
     
     virtual bool
     SetData (DataExtractor &data, Error &error);
@@ -1122,7 +1134,7 @@ protected:
     ClearDynamicTypeInformation ();
     
     //------------------------------------------------------------------
-    // Sublasses must implement the functions below.
+    // Subclasses must implement the functions below.
     //------------------------------------------------------------------
     
     virtual ClangASTType
