@@ -32,10 +32,24 @@ namespace lldb_private {
     {
     protected:
         ValueObject &m_backend;
+        
+        void
+        SetValid (bool valid)
+        {
+            m_valid = valid;
+        }
+        
+        bool
+        IsValid ()
+        {
+            return m_valid;
+        }
+        
     public:
         
         SyntheticChildrenFrontEnd (ValueObject &backend) :
-        m_backend(backend)
+        m_backend(backend),
+        m_valid(true)
         {}
         
         virtual
@@ -71,6 +85,7 @@ namespace lldb_private {
         typedef std::unique_ptr<SyntheticChildrenFrontEnd> AutoPointer;
         
     private:
+        bool m_valid;
         DISALLOW_COPY_AND_ASSIGN(SyntheticChildrenFrontEnd);
     };
     
