@@ -88,7 +88,8 @@ public:
     ClangFunction (ExecutionContextScope &exe_scope,
                    Function &function_ptr, 
                    ClangASTContext *ast_context, 
-                   const ValueList &arg_value_list);
+                   const ValueList &arg_value_list,
+                   const char *name);
     
     //------------------------------------------------------------------
     /// Constructor
@@ -114,7 +115,8 @@ public:
     ClangFunction (ExecutionContextScope &exe_scope,
                    const ClangASTType &return_type,
                    const Address& function_address, 
-                   const ValueList &arg_value_list);
+                   const ValueList &arg_value_list,
+                   const char *name);
     
     //------------------------------------------------------------------
     /// Destructor
@@ -251,9 +253,9 @@ public:
     ///     The result value will be put here after running the function.
     ///
     /// @return
-    ///     Returns one of the ExecutionResults enum indicating function call status.
+    ///     Returns one of the ExpressionResults enum indicating function call status.
     //------------------------------------------------------------------
-    ExecutionResults 
+    lldb::ExpressionResults
     ExecuteFunction(ExecutionContext &exe_ctx, 
                     lldb::addr_t *args_addr_ptr, 
                     const EvaluateExpressionOptions &options,
@@ -300,7 +302,7 @@ public:
     /// @param[in] args_addr
     ///     The address of the argument struct.
     ///
-    /// @param[in] ret_value
+    /// @param[out] ret_value
     ///     The value returned by the function.
     ///
     /// @return
@@ -410,7 +412,9 @@ private:
     //------------------------------------------------------------------
 
     std::unique_ptr<ClangExpressionParser> m_parser;                 ///< The parser responsible for compiling the function.
-    std::unique_ptr<IRExecutionUnit> m_execution_unit_ap;
+    std::shared_ptr<IRExecutionUnit> m_execution_unit_sp;
+    lldb::ModuleWP                  m_jit_module_wp;
+    std::string                     m_name;                         ///< The name of this clang function - for debugging purposes.
     
     Function                       *m_function_ptr;                 ///< The function we're going to call.  May be NULL if we don't have debug info for the function.
     Address                         m_function_addr;                ///< If we don't have the FunctionSP, we at least need the address & return type.
