@@ -32,6 +32,10 @@ namespace lldb_private {
 class Host
 {
 public:
+
+    /// A value of std::numeric_limits<uint32_t>::max() is used if there is no practical limit.
+    static const uint32_t MAX_THREAD_NAME_LENGTH;
+
     typedef bool (*MonitorChildProcessCallback) (void *callback_baton,
                                                  lldb::pid_t pid,
                                                  bool exited,
@@ -48,11 +52,11 @@ public:
     /// thread so the callback function must be thread safe.
     ///
     /// When the callback gets called, the return value indicates if
-    /// minotoring should stop. If \b true is returned from \a callback
+    /// monitoring should stop. If \b true is returned from \a callback
     /// the information will be removed. If \b false is returned then
     /// monitoring will continue. If the child process exits, the
     /// monitoring will automatically stop after the callback returned
-    /// ragardless of the callback return value.
+    /// regardless of the callback return value.
     ///
     /// @param[in] callback
     ///     A function callback to call when a child receives a signal
@@ -201,6 +205,23 @@ public:
     static const ConstString &
     GetTargetTriple ();
 
+    //------------------------------------------------------------------
+    /// Gets the name of the distribution (i.e. distributor id).
+    ///
+    /// On Linux, this will return the equivalent of lsb_release -i.
+    /// Android will return 'android'.  Other systems may return
+    /// nothing.
+    ///
+    /// @return
+    ///     A ConstString reference containing the OS distribution id.
+    ///     The return string will be all lower case, with whitespace
+    ///     replaced with underscores.  The return string will be
+    ///     empty (result.AsCString() will return NULL) if the distribution
+    ///     cannot be obtained.
+    //------------------------------------------------------------------
+    static const ConstString &
+    GetDistributionId ();
+
     //------------------------------------------------------------------
     /// Get the process ID for the calling process.
     ///
@@ -242,7 +263,7 @@ public:
     ///
     /// This function call lets the current host OS do any thread
     /// specific initialization that it needs, including naming the
-    /// thread. No cleanup routine is exptected to be called
+    /// thread. No cleanup routine is expected to be called
     ///
     /// @param[in] name
     ///     The current thread's name in the current process.
@@ -393,7 +414,7 @@ public:
     //------------------------------------------------------------------
     /// When executable files may live within a directory, where the 
     /// directory represents an executable bundle (like the MacOSX 
-    /// app bundles), the locate the executable within the containing
+    /// app bundles), then locate the executable within the containing
     /// bundle.
     ///
     /// @param[in,out] file
@@ -421,7 +442,7 @@ public:
     ///     directory member gets filled in.
     ///
     /// @param[in] file_spec
-    ///     A file spec that gets filled in with the appriopriate path.
+    ///     A file spec that gets filled in with the appropriate path.
     ///
     /// @return
     ///     \b true if \a resource_path was resolved, \a false otherwise.
@@ -459,7 +480,15 @@ public:
 
     static bool
     GetProcessInfo (lldb::pid_t pid, ProcessInstanceInfo &proc_info);
-    
+
+#if defined (__APPLE__) || defined (__linux__) || defined (__FreeBSD__) || defined (__GLIBC__) || defined (__NetBSD__)
+    static short
+    GetPosixspawnFlags (ProcessLaunchInfo &launch_info);
+
+    static Error
+    LaunchProcessPosixSpawn (const char *exe_path, ProcessLaunchInfo &launch_info, ::pid_t &pid);
+#endif
+
     static lldb::pid_t
     LaunchApplication (const FileSpec &app_file_spec);
 
@@ -478,6 +507,9 @@ public:
     static lldb::DataBufferSP
     GetAuxvData (lldb_private::Process *process);
 
+    static lldb::DataBufferSP
+    GetAuxvData (lldb::pid_t pid);
+
     static lldb::TargetSP
     GetDummyTarget (Debugger &debugger);
     
@@ -510,13 +542,31 @@ public:
                              const char *symbol_name, 
                              Error &error);
     
-    static uint32_t
-    MakeDirectory (const char* path, mode_t mode);
+    static Error
+    MakeDirectory (const char* path, uint32_t mode);
     
+    static Error
+    RemoveDirectory (const char* path, bool recurse);
+    
+    static Error
+    GetFilePermissions (const char* path, uint32_t &file_permissions);
+
+    static Error
+    SetFilePermissions (const char* path, uint32_t file_permissions);
+    
+    static Error
+    Symlink (const char *src, const char *dst);
+    
+    static Error
+    Readlink (const char *path, char *buf, size_t buf_len);
+
+    static Error
+    Unlink (const char *path);
+
     static lldb::user_id_t
     OpenFile (const FileSpec& file_spec,
               uint32_t flags,
-              mode_t mode,
+              uint32_t mode,
               Error &error);
     
     static bool
