@@ -17,6 +17,7 @@
 #include "lldb/Core/Address.h"
 #include "lldb/Core/Mangled.h"
 #include "lldb/Symbol/LineEntry.h"
+#include "lldb/Utility/Iterable.h"
 
 namespace lldb_private {
 
@@ -139,7 +140,7 @@ public:
     /// supplied stream \a s.
     ///
     /// @param[in] s
-    ///     The stream to which to dump the object descripton.
+    ///     The stream to which to dump the object description.
     //------------------------------------------------------------------
     void
     Dump (Stream *s, Target *target) const;
@@ -156,7 +157,7 @@ public:
     /// was stopped will be displayed.
     ///
     /// @param[in] s
-    ///     The stream to which to dump the object descripton.
+    ///     The stream to which to dump the object description.
     ///
     /// @param[in] so_addr
     ///     The resolved section offset address.
@@ -408,7 +409,7 @@ private:
 /// the result of a query that can contain a multiple results. Examples
 /// of such queries include:
 ///     @li Looking up a function by name.
-///     @li Finding all addressses for a specified file and line number.
+///     @li Finding all addresses for a specified file and line number.
 //----------------------------------------------------------------------
 class SymbolContextList
 {
@@ -464,7 +465,7 @@ public:
     /// the list to the supplied stream \a s.
     ///
     /// @param[in] s
-    ///     The stream to which to dump the object descripton.
+    ///     The stream to which to dump the object description.
     //------------------------------------------------------------------
     void
     Dump(Stream *s, Target *target) const;
@@ -552,6 +553,14 @@ protected:
     // Member variables.
     //------------------------------------------------------------------
     collection m_symbol_contexts; ///< The list of symbol contexts.
+
+public:
+    typedef AdaptedIterable<collection, SymbolContext, vector_adapter> SymbolContextIterable;
+    SymbolContextIterable
+    SymbolContexts()
+    {
+        return SymbolContextIterable(m_symbol_contexts);
+    }
 };
 
 bool operator== (const SymbolContext& lhs, const SymbolContext& rhs);
