@@ -69,7 +69,7 @@ SBModule::SBModule (lldb::SBProcess &process, lldb::addr_t header_addr) :
         {
             Target &target = process_sp->GetTarget();
             bool changed = false;
-            m_opaque_sp->SetLoadAddress(target, 0, changed);
+            m_opaque_sp->SetLoadAddress(target, 0, true, changed);
             target.GetImages().Append(m_opaque_sp);
         }
     }
@@ -110,10 +110,9 @@ SBModule::GetFileSpec () const
         file_spec.SetFileSpec(module_sp->GetFileSpec());
 
     if (log)
-    {
-        log->Printf ("SBModule(%p)::GetFileSpec () => SBFileSpec(%p)", 
-        module_sp.get(), file_spec.get());
-    }
+        log->Printf ("SBModule(%p)::GetFileSpec () => SBFileSpec(%p)",
+                     static_cast<void*>(module_sp.get()),
+                     static_cast<const void*>(file_spec.get()));
 
     return file_spec;
 }
@@ -122,20 +121,18 @@ lldb::SBFileSpec
 SBModule::GetPlatformFileSpec () const
 {
     Log *log(lldb_private::GetLogIfAllCategoriesSet (LIBLLDB_LOG_API));
-    
+
     SBFileSpec file_spec;
     ModuleSP module_sp (GetSP ());
     if (module_sp)
         file_spec.SetFileSpec(module_sp->GetPlatformFileSpec());
-    
+
     if (log)
-    {
-        log->Printf ("SBModule(%p)::GetPlatformFileSpec () => SBFileSpec(%p)", 
-                     module_sp.get(), file_spec.get());
-    }
-    
+        log->Printf ("SBModule(%p)::GetPlatformFileSpec () => SBFileSpec(%p)",
+                     static_cast<void*>(module_sp.get()),
+                     static_cast<const void*>(file_spec.get()));
+
     return file_spec;
-    
 }
 
 bool
@@ -143,25 +140,43 @@ SBModule::SetPlatformFileSpec (const lldb::SBFileSpec &platform_file)
 {
     bool result = false;
     Log *log(lldb_private::GetLogIfAllCategoriesSet (LIBLLDB_LOG_API));
-    
+
     ModuleSP module_sp (GetSP ());
     if (module_sp)
     {
         module_sp->SetPlatformFileSpec(*platform_file);
         result = true;
     }
-    
+
     if (log)
-    {
-        log->Printf ("SBModule(%p)::SetPlatformFileSpec (SBFileSpec(%p (%s)) => %i", 
-                     module_sp.get(), 
-                     platform_file.get(),
-                     platform_file->GetPath().c_str(),
-                     result);
-    }
+        log->Printf ("SBModule(%p)::SetPlatformFileSpec (SBFileSpec(%p (%s)) => %i",
+                     static_cast<void*>(module_sp.get()),
+                     static_cast<const void*>(platform_file.get()),
+                     platform_file->GetPath().c_str(), result);
     return result;
 }
 
+lldb::SBFileSpec
+SBModule::GetRemoteInstallFileSpec ()
+{
+    SBFileSpec sb_file_spec;
+    ModuleSP module_sp (GetSP ());
+    if (module_sp)
+        sb_file_spec.SetFileSpec (module_sp->GetRemoteInstallFileSpec());
+    return sb_file_spec;
+}
+
+bool
+SBModule::SetRemoteInstallFileSpec (lldb::SBFileSpec &file)
+{
+    ModuleSP module_sp (GetSP ());
+    if (module_sp)
+    {
+        module_sp->SetRemoteInstallFileSpec(file.ref());
+        return true;
+    }
+    return false;
+}
 
 
 const uint8_t *
@@ -180,10 +195,12 @@ SBModule::GetUUIDBytes () const
         {
             StreamString s;
             module_sp->GetUUID().Dump (&s);
-            log->Printf ("SBModule(%p)::GetUUIDBytes () => %s", module_sp.get(), s.GetData());
+            log->Printf ("SBModule(%p)::GetUUIDBytes () => %s",
+                         static_cast<void*>(module_sp.get()), s.GetData());
         }
         else
-            log->Printf ("SBModule(%p)::GetUUIDBytes () => NULL", module_sp.get());
+            log->Printf ("SBModule(%p)::GetUUIDBytes () => NULL",
+                         static_cast<void*>(module_sp.get()));
     }
     return uuid_bytes;
 }
@@ -213,10 +230,12 @@ SBModule::GetUUIDString () const
         {
             StreamString s;
             module_sp->GetUUID().Dump (&s);
-            log->Printf ("SBModule(%p)::GetUUIDString () => %s", module_sp.get(), s.GetData());
+            log->Printf ("SBModule(%p)::GetUUIDString () => %s",
+                         static_cast<void*>(module_sp.get()), s.GetData());
         }
         else
-            log->Printf ("SBModule(%p)::GetUUIDString () => NULL", module_sp.get());
+            log->Printf ("SBModule(%p)::GetUUIDString () => NULL",
+                         static_cast<void*>(module_sp.get()));
     }
     return uuid_c_string;
 }
@@ -558,6 +577,23 @@ SBModule::FindTypes (const char *type)
     return retval;
 }
 
+lldb::SBType
+SBModule::GetTypeByID (lldb::user_id_t uid)
+{
+    ModuleSP module_sp (GetSP ());
+    if (module_sp)
+    {
+        SymbolVendor* vendor = module_sp->GetSymbolVendor();
+        if (vendor)
+        {
+            Type *type_ptr = vendor->ResolveTypeUID(uid);
+            if (type_ptr)
+                return SBType(type_ptr->shared_from_this());
+        }
+    }
+    return SBType();
+}
+
 lldb::SBTypeList
 SBModule::GetTypes (uint32_t type_mask)
 {
