@@ -153,7 +153,7 @@ LoadPluginCallback
     {
         // Try and recurse into anything that a directory or symbolic link. 
         // We must also do this for unknown as sometimes the directory enumeration
-        // might be enurating a file system that doesn't have correct file type
+        // might be enumerating a file system that doesn't have correct file type
         // information.
         return FileSpec::eEnumerateDirectoryResultEnter;
     }
@@ -544,6 +544,116 @@ PluginManager::GetDynamicLoaderCreateCallbackForPluginName (const ConstString &n
     return NULL;
 }
 
+#pragma mark JITLoader
+
+
+struct JITLoaderInstance
+{
+    JITLoaderInstance() :
+        name(),
+        description(),
+        create_callback(NULL),
+        debugger_init_callback (NULL)
+    {
+    }
+
+    ConstString name;
+    std::string description;
+    JITLoaderCreateInstance create_callback;
+    DebuggerInitializeCallback debugger_init_callback;
+};
+
+typedef std::vector<JITLoaderInstance> JITLoaderInstances;
+
+
+static Mutex &
+GetJITLoaderMutex ()
+{
+    static Mutex g_instances_mutex (Mutex::eMutexTypeRecursive);
+    return g_instances_mutex;
+}
+
+static JITLoaderInstances &
+GetJITLoaderInstances ()
+{
+    static JITLoaderInstances g_instances;
+    return g_instances;
+}
+
+
+bool
+PluginManager::RegisterPlugin
+(
+    const ConstString &name,
+    const char *description,
+    JITLoaderCreateInstance create_callback,
+    DebuggerInitializeCallback debugger_init_callback
+)
+{
+    if (create_callback)
+    {
+        JITLoaderInstance instance;
+        assert ((bool)name);
+        instance.name = name;
+        if (description && description[0])
+            instance.description = description;
+        instance.create_callback = create_callback;
+        instance.debugger_init_callback = debugger_init_callback;
+        Mutex::Locker locker (GetJITLoaderMutex ());
+        GetJITLoaderInstances ().push_back (instance);
+    }
+    return false;
+}
+
+bool
+PluginManager::UnregisterPlugin (JITLoaderCreateInstance create_callback)
+{
+    if (create_callback)
+    {
+        Mutex::Locker locker (GetJITLoaderMutex ());
+        JITLoaderInstances &instances = GetJITLoaderInstances ();
+        
+        JITLoaderInstances::iterator pos, end = instances.end();
+        for (pos = instances.begin(); pos != end; ++ pos)
+        {
+            if (pos->create_callback == create_callback)
+            {
+                instances.erase(pos);
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+JITLoaderCreateInstance
+PluginManager::GetJITLoaderCreateCallbackAtIndex (uint32_t idx)
+{
+    Mutex::Locker locker (GetJITLoaderMutex ());
+    JITLoaderInstances &instances = GetJITLoaderInstances ();
+    if (idx < instances.size())
+        return instances[idx].create_callback;
+    return NULL;
+}
+
+JITLoaderCreateInstance
+PluginManager::GetJITLoaderCreateCallbackForPluginName (const ConstString &name)
+{
+    if (name)
+    {
+        Mutex::Locker locker (GetJITLoaderMutex ());
+        JITLoaderInstances &instances = GetJITLoaderInstances ();
+        
+        JITLoaderInstances::iterator pos, end = instances.end();
+        for (pos = instances.begin(); pos != end; ++ pos)
+        {
+            if (name == pos->name)
+                return pos->create_callback;
+        }
+    }
+    return NULL;
+}
+
 #pragma mark EmulateInstruction
 
 
@@ -968,7 +1078,8 @@ struct ObjectFileInstance
         description(),
         create_callback(NULL),
         create_memory_callback (NULL),
-        get_module_specifications (NULL)
+        get_module_specifications (NULL),
+        save_core (NULL)
     {
     }
 
@@ -977,6 +1088,7 @@ struct ObjectFileInstance
     ObjectFileCreateInstance create_callback;
     ObjectFileCreateMemoryInstance create_memory_callback;
     ObjectFileGetModuleSpecifications get_module_specifications;
+    ObjectFileSaveCore save_core;
 };
 
 typedef std::vector<ObjectFileInstance> ObjectFileInstances;
@@ -1001,7 +1113,8 @@ PluginManager::RegisterPlugin (const ConstString &name,
                                const char *description,
                                ObjectFileCreateInstance create_callback,
                                ObjectFileCreateMemoryInstance create_memory_callback,
-                               ObjectFileGetModuleSpecifications get_module_specifications)
+                               ObjectFileGetModuleSpecifications get_module_specifications,
+                               ObjectFileSaveCore save_core)
 {
     if (create_callback)
     {
@@ -1012,6 +1125,7 @@ PluginManager::RegisterPlugin (const ConstString &name,
             instance.description = description;
         instance.create_callback = create_callback;
         instance.create_memory_callback = create_memory_callback;
+        instance.save_core = save_core;
         instance.get_module_specifications = get_module_specifications;
         Mutex::Locker locker (GetObjectFileMutex ());
         GetObjectFileInstances ().push_back (instance);
@@ -1108,7 +1222,22 @@ PluginManager::GetObjectFileCreateMemoryCallbackForPluginName (const ConstString
     return NULL;
 }
 
-
+Error
+PluginManager::SaveCore (const lldb::ProcessSP &process_sp, const FileSpec &outfile)
+{
+    Error error;
+    Mutex::Locker locker (GetObjectFileMutex ());
+    ObjectFileInstances &instances = GetObjectFileInstances ();
+    
+    ObjectFileInstances::iterator pos, end = instances.end();
+    for (pos = instances.begin(); pos != end; ++ pos)
+    {
+        if (pos->save_core && pos->save_core (process_sp, outfile, error))
+            return error;
+    }
+    error.SetErrorString("no ObjectFile plugins were able to save a core for this process");
+    return error;
+}
 
 #pragma mark ObjectContainer
 
@@ -1945,6 +2074,19 @@ PluginManager::DebuggerInitialize (Debugger &debugger)
         }
     }
 
+    // Initialize the JITLoader plugins
+    {
+        Mutex::Locker locker (GetJITLoaderMutex ());
+        JITLoaderInstances &instances = GetJITLoaderInstances ();
+    
+        JITLoaderInstances::iterator pos, end = instances.end();
+        for (pos = instances.begin(); pos != end; ++ pos)
+        {
+            if (pos->debugger_init_callback)
+                pos->debugger_init_callback (debugger);
+        }
+    }
+
     // Initialize the Platform plugins
     {
         Mutex::Locker locker (GetPlatformInstancesMutex ());
