@@ -342,8 +342,7 @@ lldb_private::formatters::NSNumberSummaryProvider (ValueObject& valobj, Stream&
                     stream.Printf("(long)%" PRId64,value);
                     break;
                 default:
-                    stream.Printf("unexpected value:(info=%" PRIu64 ", value=%" PRIu64,i_bits,value);
-                    break;
+                    return false;
             }
             return true;
         }
@@ -402,8 +401,7 @@ lldb_private::formatters::NSNumberSummaryProvider (ValueObject& valobj, Stream&
                     break;
                 }
                 default:
-                    stream.Printf("unexpected value: dt=%d",data_type);
-                    break;
+                    return false;
             }
             return true;
         }
