@@ -1,4 +1,4 @@
-//===-- PlatformFreeBSD.cpp ---------------------------------------*- C++ -*-===//
+//===-- PlatformFreeBSD.cpp -------------------------------------*- C++ -*-===//
 //
 //                     The LLVM Compiler Infrastructure
 //
@@ -47,10 +47,10 @@ PlatformFreeBSD::CreateInstance (bool force, const lldb_private::ArchSpec *arch)
             case llvm::Triple::PC:
                 create = true;
                 break;
-                
+
 #if defined(__FreeBSD__) || defined(__OpenBSD__)
             // Only accept "unknown" for the vendor if the host is BSD and
-            // it "unknown" wasn't specified (it was just returned becasue it
+            // it "unknown" wasn't specified (it was just returned because it
             // was NOT specified)
             case llvm::Triple::UnknownArch:
                 create = !arch->TripleVendorWasSpecified();
@@ -59,7 +59,7 @@ PlatformFreeBSD::CreateInstance (bool force, const lldb_private::ArchSpec *arch)
             default:
                 break;
         }
-        
+
         if (create)
         {
             switch (triple.getOS())
@@ -67,10 +67,10 @@ PlatformFreeBSD::CreateInstance (bool force, const lldb_private::ArchSpec *arch)
                 case llvm::Triple::FreeBSD:
                 case llvm::Triple::KFreeBSD:
                     break;
-                    
+
 #if defined(__FreeBSD__) || defined(__OpenBSD__)
                 // Only accept "unknown" for the OS if the host is BSD and
-                // it "unknown" wasn't specified (it was just returned becasue it
+                // it "unknown" wasn't specified (it was just returned because it
                 // was NOT specified)
                 case llvm::Triple::UnknownOS:
                     create = arch->TripleOSWasSpecified();
@@ -142,7 +142,8 @@ PlatformFreeBSD::Terminate ()
 /// Default Constructor
 //------------------------------------------------------------------
 PlatformFreeBSD::PlatformFreeBSD (bool is_host) :
-Platform(is_host)
+Platform(is_host),
+m_remote_platform_sp()
 {
 }
 
@@ -185,10 +186,10 @@ PlatformFreeBSD::ResolveExecutable (const FileSpec &exe_file,
 {
     Error error;
     // Nothing special to do here, just use the actual file and architecture
-    
+
     char exe_path[PATH_MAX];
     FileSpec resolved_exe_file (exe_file);
-    
+
     if (IsHost())
     {
         // If we have "ls" as the exe_file, resolve the executable location based on
@@ -198,10 +199,10 @@ PlatformFreeBSD::ResolveExecutable (const FileSpec &exe_file,
             exe_file.GetPath(exe_path, sizeof(exe_path));
             resolved_exe_file.SetFile(exe_path, true);
         }
-        
+
         if (!resolved_exe_file.Exists())
             resolved_exe_file.ResolveExecutableLocation ();
-        
+
         if (resolved_exe_file.Exists())
             error.Clear();
         else
@@ -222,10 +223,10 @@ PlatformFreeBSD::ResolveExecutable (const FileSpec &exe_file,
         else
         {
             // We may connect to a process and use the provided executable (Don't use local $PATH).
-            
+
             // Resolve any executable within a bundle on MacOSX
             Host::ResolveExecutableInBundle (resolved_exe_file);
-            
+
             if (resolved_exe_file.Exists()) {
                 error.Clear();
             }
@@ -247,7 +248,7 @@ PlatformFreeBSD::ResolveExecutable (const FileSpec &exe_file,
                                                  module_search_paths_ptr,
                                                  NULL,
                                                  NULL);
-            
+
             if (!exe_module_sp || exe_module_sp->GetObjectFile() == NULL)
             {
                 exe_module_sp.reset();
@@ -278,12 +279,12 @@ PlatformFreeBSD::ResolveExecutable (const FileSpec &exe_file,
                     else
                         error.SetErrorToGenericError();
                 }
-                
+
                 if (idx > 0)
                     arch_names.PutCString (", ");
                 arch_names.PutCString (platform_arch.GetArchitectureName());
             }
-            
+
             if (error.Fail() || !exe_module_sp)
             {
                 error.SetErrorStringWithFormat ("'%s' doesn't contain any '%s' platform architectures: %s",
@@ -304,14 +305,13 @@ PlatformFreeBSD::GetSoftwareBreakpointTrapOpcode (Target &target, BreakpointSite
     const uint8_t *trap_opcode = NULL;
     size_t trap_opcode_size = 0;
 
-    switch (arch.GetCore())
+    switch (arch.GetMachine())
     {
     default:
         assert(false && "Unhandled architecture in PlatformFreeBSD::GetSoftwareBreakpointTrapOpcode()");
         break;
-
-    case ArchSpec::eCore_x86_32_i386:
-    case ArchSpec::eCore_x86_64_x86_64:
+    case llvm::Triple::x86:
+    case llvm::Triple::x86_64:
         {
             static const uint8_t g_i386_opcode[] = { 0xCC };
             trap_opcode = g_i386_opcode;
@@ -449,7 +449,7 @@ PlatformFreeBSD::GetProcessInfo (lldb::pid_t pid, ProcessInstanceInfo &process_i
     {
         success = Platform::GetProcessInfo (pid, process_info);
     }
-    else if (m_remote_platform_sp) 
+    else if (m_remote_platform_sp)
     {
         success = m_remote_platform_sp->GetProcessInfo (pid, process_info);
     }
@@ -571,14 +571,14 @@ PlatformFreeBSD::GetGroupName (uint32_t gid)
 
 // From PlatformMacOSX only
 Error
-PlatformFreeBSD::GetFile (const FileSpec &platform_file,
-                          const UUID *uuid_ptr,
-                          FileSpec &local_file)
+PlatformFreeBSD::GetFileWithUUID (const FileSpec &platform_file,
+                                  const UUID *uuid_ptr,
+                                  FileSpec &local_file)
 {
     if (IsRemote())
     {
         if (m_remote_platform_sp)
-            return m_remote_platform_sp->GetFile (platform_file, uuid_ptr, local_file);
+            return m_remote_platform_sp->GetFileWithUUID (platform_file, uuid_ptr, local_file);
     }
 
     // Default to the local case
@@ -673,3 +673,9 @@ PlatformFreeBSD::GetStatus (Stream &strm)
 
     Platform::GetStatus(strm);
 }
+
+void
+PlatformFreeBSD::CalculateTrapHandlerSymbolNames ()
+{
+    m_trap_handlers.push_back (ConstString ("_sigtramp"));
+}
