@@ -28,6 +28,7 @@
 #include "lldb/Symbol/SymbolFile.h"
 #include "lldb/Symbol/SymbolVendor.h"
 #include "lldb/Target/Target.h"
+#include "llvm/ADT/STLExtras.h"
 
 using namespace lldb;
 using namespace lldb_private;
@@ -38,8 +39,7 @@ using namespace lldb_private;
 //------------------------------------------------------------------
 PlatformDarwin::PlatformDarwin (bool is_host) :
     PlatformPOSIX(is_host),  // This is the local host platform
-    m_developer_directory (),
-    m_dispatch_queue_offsets_addr (LLDB_INVALID_ADDRESS)
+    m_developer_directory ()
 {
 }
 
@@ -273,14 +273,7 @@ static lldb_private::Error
 MakeCacheFolderForFile (const FileSpec& module_cache_spec)
 {
     FileSpec module_cache_folder = module_cache_spec.CopyByRemovingLastPathComponent();
-    StreamString mkdir_folder_cmd;
-    mkdir_folder_cmd.Printf("mkdir -p %s/%s", module_cache_folder.GetDirectory().AsCString(), module_cache_folder.GetFilename().AsCString());
-    return Host::RunShellCommand(mkdir_folder_cmd.GetData(),
-                          NULL,
-                          NULL,
-                          NULL,
-                          NULL,
-                          60);
+    return Host::MakeDirectory(module_cache_folder.GetPath().c_str(), eFilePermissionsDirectoryDefault);
 }
 
 static lldb_private::Error
@@ -303,7 +296,7 @@ PlatformDarwin::GetSharedModuleWithLocalCache (const lldb_private::ModuleSpec &m
 
     Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
     if (log)
-        log->Printf("[%s] Trying to find module %s/%s - platform path %s/%s symbol path %s/%s\n",
+        log->Printf("[%s] Trying to find module %s/%s - platform path %s/%s symbol path %s/%s",
                      (IsHost() ? "host" : "remote"),
                      module_spec.GetFileSpec().GetDirectory().AsCString(),
                      module_spec.GetFileSpec().GetFilename().AsCString(),
@@ -311,102 +304,113 @@ PlatformDarwin::GetSharedModuleWithLocalCache (const lldb_private::ModuleSpec &m
                      module_spec.GetPlatformFileSpec().GetFilename().AsCString(),
                      module_spec.GetSymbolFileSpec().GetDirectory().AsCString(),
                      module_spec.GetSymbolFileSpec().GetFilename().AsCString());
-
-    std::string cache_path(GetLocalCacheDirectory());
-    std::string module_path (module_spec.GetFileSpec().GetPath());
-    cache_path.append(module_path);
-    FileSpec module_cache_spec(cache_path.c_str(),false);
     
-    // if rsync is supported, always bring in the file - rsync will be very efficient
-    // when files are the same on the local and remote end of the connection
-    if (this->GetSupportsRSync())
+    Error err;
+    
+    err = ModuleList::GetSharedModule(module_spec, module_sp, module_search_paths_ptr, old_module_sp_ptr, did_create_ptr);
+    if (module_sp)
+        return err;
+    
+    if (!IsHost())
     {
-        Error err = BringInRemoteFile (this, module_spec, module_cache_spec);
-        if (err.Fail())
-            return err;
-        if (module_cache_spec.Exists())
+        std::string cache_path(GetLocalCacheDirectory());
+        // Only search for a locally cached file if we have a valid cache path
+        if (!cache_path.empty())
         {
-            Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
+            std::string module_path (module_spec.GetFileSpec().GetPath());
+            cache_path.append(module_path);
+            FileSpec module_cache_spec(cache_path.c_str(),false);
+        
+            // if rsync is supported, always bring in the file - rsync will be very efficient
+            // when files are the same on the local and remote end of the connection
+            if (this->GetSupportsRSync())
+            {
+                err = BringInRemoteFile (this, module_spec, module_cache_spec);
+                if (err.Fail())
+                    return err;
+                if (module_cache_spec.Exists())
+                {
+                    Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
+                    if (log)
+                        log->Printf("[%s] module %s/%s was rsynced and is now there",
+                                     (IsHost() ? "host" : "remote"),
+                                     module_spec.GetFileSpec().GetDirectory().AsCString(),
+                                     module_spec.GetFileSpec().GetFilename().AsCString());
+                    ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
+                    module_sp.reset(new Module(local_spec));
+                    module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
+                    return Error();
+                }
+            }
+            
+            // try to find the module in the cache
+            if (module_cache_spec.Exists())
+            {
+                // get the local and remote MD5 and compare
+                if (m_remote_platform_sp)
+                {
+                    // when going over the *slow* GDB remote transfer mechanism we first check
+                    // the hashes of the files - and only do the actual transfer if they differ
+                    uint64_t high_local,high_remote,low_local,low_remote;
+                    Host::CalculateMD5 (module_cache_spec, low_local, high_local);
+                    m_remote_platform_sp->CalculateMD5(module_spec.GetFileSpec(), low_remote, high_remote);
+                    if (low_local != low_remote || high_local != high_remote)
+                    {
+                        // bring in the remote file
+                        Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
+                        if (log)
+                            log->Printf("[%s] module %s/%s needs to be replaced from remote copy",
+                                         (IsHost() ? "host" : "remote"),
+                                         module_spec.GetFileSpec().GetDirectory().AsCString(),
+                                         module_spec.GetFileSpec().GetFilename().AsCString());
+                        Error err = BringInRemoteFile (this, module_spec, module_cache_spec);
+                        if (err.Fail())
+                            return err;
+                    }
+                }
+                
+                ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
+                module_sp.reset(new Module(local_spec));
+                module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
+                Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
+                    if (log)
+                        log->Printf("[%s] module %s/%s was found in the cache",
+                                     (IsHost() ? "host" : "remote"),
+                                     module_spec.GetFileSpec().GetDirectory().AsCString(),
+                                     module_spec.GetFileSpec().GetFilename().AsCString());
+                return Error();
+            }
+            
+            // bring in the remote module file
             if (log)
-                log->Printf("[%s] module %s/%s was rsynced and is now there\n",
+                log->Printf("[%s] module %s/%s needs to come in remotely",
                              (IsHost() ? "host" : "remote"),
                              module_spec.GetFileSpec().GetDirectory().AsCString(),
                              module_spec.GetFileSpec().GetFilename().AsCString());
-            ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
-            module_sp.reset(new Module(local_spec));
-            module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
-            return Error();
-        }
-    }
-
-    if (module_spec.GetFileSpec().Exists() && !module_sp)
-    {
-        module_sp.reset(new Module(module_spec));
-        return Error();
-    }
-    
-    // try to find the module in the cache
-    if (module_cache_spec.Exists())
-    {
-        // get the local and remote MD5 and compare
-        if (m_remote_platform_sp)
-        {
-            // when going over the *slow* GDB remote transfer mechanism we first check
-            // the hashes of the files - and only do the actual transfer if they differ
-            uint64_t high_local,high_remote,low_local,low_remote;
-            Host::CalculateMD5 (module_cache_spec, low_local, high_local);
-            m_remote_platform_sp->CalculateMD5(module_spec.GetFileSpec(), low_remote, high_remote);
-            if (low_local != low_remote || high_local != high_remote)
+            Error err = BringInRemoteFile (this, module_spec, module_cache_spec);
+            if (err.Fail())
+                return err;
+            if (module_cache_spec.Exists())
             {
-                // bring in the remote file
                 Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
                 if (log)
-                    log->Printf("[%s] module %s/%s needs to be replaced from remote copy\n",
+                    log->Printf("[%s] module %s/%s is now cached and fine",
                                  (IsHost() ? "host" : "remote"),
                                  module_spec.GetFileSpec().GetDirectory().AsCString(),
                                  module_spec.GetFileSpec().GetFilename().AsCString());
-                Error err = BringInRemoteFile (this, module_spec, module_cache_spec);
-                if (err.Fail())
-                    return err;
+                ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
+                module_sp.reset(new Module(local_spec));
+                module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
+                return Error();
             }
+            else
+                return Error("unable to obtain valid module file");
         }
-        
-        ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
-        module_sp.reset(new Module(local_spec));
-        module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
-        Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
-            if (log)
-                log->Printf("[%s] module %s/%s was found in the cache\n",
-                             (IsHost() ? "host" : "remote"),
-                             module_spec.GetFileSpec().GetDirectory().AsCString(),
-                             module_spec.GetFileSpec().GetFilename().AsCString());
-        return Error();
-    }
-    
-    // bring in the remote module file
-    if (log)
-        log->Printf("[%s] module %s/%s needs to come in remotely\n",
-                     (IsHost() ? "host" : "remote"),
-                     module_spec.GetFileSpec().GetDirectory().AsCString(),
-                     module_spec.GetFileSpec().GetFilename().AsCString());
-    Error err = BringInRemoteFile (this, module_spec, module_cache_spec);
-    if (err.Fail())
-        return err;
-    if (module_cache_spec.Exists())
-    {
-        Log *log(GetLogIfAnyCategoriesSet(LIBLLDB_LOG_PLATFORM));
-        if (log)
-            log->Printf("[%s] module %s/%s is now cached and fine\n",
-                         (IsHost() ? "host" : "remote"),
-                         module_spec.GetFileSpec().GetDirectory().AsCString(),
-                         module_spec.GetFileSpec().GetFilename().AsCString());
-        ModuleSpec local_spec(module_cache_spec, module_spec.GetArchitecture());
-        module_sp.reset(new Module(local_spec));
-        module_sp->SetPlatformFileSpec(module_spec.GetFileSpec());
-        return Error();
+        else
+            return Error("no cache path");
     }
     else
-        return Error("unable to obtain valid module file");
+        return Error ("unable to resolve module");
 }
 
 Error
@@ -527,6 +531,17 @@ PlatformDarwin::GetSoftwareBreakpointTrapOpcode (Target &target, BreakpointSite
         }
         break;
 
+    case llvm::Triple::arm64:
+        {
+            // TODO: fix this with actual darwin breakpoint opcode for arm64.
+            // right now debugging uses the Z packets with GDB remote so this
+            // is not needed, but the size needs to be correct...
+            static const uint8_t g_arm64_breakpoint_opcode[] = { 0xFE, 0xDE, 0xFF, 0xE7 };
+            trap_opcode = g_arm64_breakpoint_opcode;
+            trap_opcode_size = sizeof(g_arm64_breakpoint_opcode);
+        }
+        break;
+
     case llvm::Triple::thumb:
         bp_is_thumb = true; // Fall through...
     case llvm::Triple::arm:
@@ -760,6 +775,35 @@ PlatformDarwin::LaunchProcess (ProcessLaunchInfo &launch_info)
     return error;
 }
 
+lldb::ProcessSP
+PlatformDarwin::DebugProcess (ProcessLaunchInfo &launch_info,
+                              Debugger &debugger,
+                              Target *target,       // Can be NULL, if NULL create a new target, else use existing one
+                              Listener &listener,
+                              Error &error)
+{
+    ProcessSP process_sp;
+    
+    if (IsHost())
+    {
+        // We are going to hand this process off to debugserver which will be in charge of setting the exit status.
+        // We still need to reap it from lldb but if we let the monitor thread also set the exit status, we set up a
+        // race between debugserver & us for who will find out about the debugged process's death.
+        launch_info.GetFlags().Set(eLaunchFlagDontSetExitStatus);
+        process_sp = Platform::DebugProcess (launch_info, debugger, target, listener, error);
+    }
+    else
+    {
+        if (m_remote_platform_sp)
+            process_sp = m_remote_platform_sp->DebugProcess (launch_info, debugger, target, listener, error);
+        else
+            error.SetErrorString ("the platform is not currently connected");
+    }
+    return process_sp;
+    
+}
+
+
 lldb::ProcessSP
 PlatformDarwin::Attach (ProcessAttachInfo &attach_info,
                         Debugger &debugger,
@@ -793,7 +837,12 @@ PlatformDarwin::Attach (ProcessAttachInfo &attach_info,
             process_sp = target->CreateProcess (listener, attach_info.GetProcessPluginName(), NULL);
             
             if (process_sp)
+            {
+                ListenerSP listener_sp (new Listener("lldb.PlatformDarwin.attach.hijack"));
+                attach_info.SetHijackListener(listener_sp);
+                process_sp->HijackProcessEvents(listener_sp.get());
                 error = process_sp->Attach (attach_info);
+            }
         }
     }
     else
@@ -848,134 +897,49 @@ PlatformDarwin::ModuleIsExcludedForNonModuleSpecificSearches (lldb_private::Targ
         return false;
 }
 
-std::string
-PlatformDarwin::GetQueueNameForThreadQAddress (Process *process, addr_t thread_dispatch_qaddr)
-{
-    std::string dispatch_queue_name;
-    if (thread_dispatch_qaddr == LLDB_INVALID_ADDRESS || thread_dispatch_qaddr == 0 || process == NULL)
-        return "";
-
-    Target &target = process->GetTarget();
-
-    // Cache the dispatch_queue_offsets_addr value so we don't always have
-    // to look it up
-    if (m_dispatch_queue_offsets_addr == LLDB_INVALID_ADDRESS)
-    {
-        static ConstString g_dispatch_queue_offsets_symbol_name ("dispatch_queue_offsets");
-        const Symbol *dispatch_queue_offsets_symbol = NULL;
-
-        // libdispatch symbols were in libSystem.B.dylib up through Mac OS X 10.6 ("Snow Leopard")
-        ModuleSpec libSystem_module_spec (FileSpec("libSystem.B.dylib", false));
-        ModuleSP module_sp(target.GetImages().FindFirstModule (libSystem_module_spec));
-        if (module_sp)
-            dispatch_queue_offsets_symbol = module_sp->FindFirstSymbolWithNameAndType (g_dispatch_queue_offsets_symbol_name, eSymbolTypeData);
-        
-        // libdispatch symbols are in their own dylib as of Mac OS X 10.7 ("Lion") and later
-        if (dispatch_queue_offsets_symbol == NULL)
-        {
-            ModuleSpec libdispatch_module_spec (FileSpec("libdispatch.dylib", false));
-            module_sp = target.GetImages().FindFirstModule (libdispatch_module_spec);
-            if (module_sp)
-                dispatch_queue_offsets_symbol = module_sp->FindFirstSymbolWithNameAndType (g_dispatch_queue_offsets_symbol_name, eSymbolTypeData);
-        }
-        if (dispatch_queue_offsets_symbol)
-            m_dispatch_queue_offsets_addr = dispatch_queue_offsets_symbol->GetAddress().GetLoadAddress(&target);
-
-        if (m_dispatch_queue_offsets_addr == LLDB_INVALID_ADDRESS)
-            return "";
-    }
-
-    uint8_t memory_buffer[8];
-    DataExtractor data (memory_buffer, 
-                        sizeof(memory_buffer), 
-                        target.GetArchitecture().GetByteOrder(), 
-                        target.GetArchitecture().GetAddressByteSize());
-
-    // Excerpt from src/queue_private.h
-    // version 4 of this struct first appears in Mac OS X 10.9 ("Mavericks") and iOS 7.
-    // TODO When version 1-3 no longer needs to be supported, the dqo_label offset should be
-    // read from the inferior one time and saved in an ivar like m_dispatch_queue_offsets_addr.
-    struct dispatch_queue_offsets_s
-    {
-        uint16_t dqo_version;
-        uint16_t dqo_label;      // in version 1-3, offset to string; in version 4+, offset to a pointer to a string
-        uint16_t dqo_label_size; // in version 1-3, length of string; in version 4+, size of a (void*) in this process
-    } dispatch_queue_offsets;
-
-    Error error;
-    if (process->ReadMemory (m_dispatch_queue_offsets_addr, memory_buffer, sizeof(dispatch_queue_offsets), error) == sizeof(dispatch_queue_offsets))
-    {
-        lldb::offset_t data_offset = 0;
-        if (data.GetU16(&data_offset, &dispatch_queue_offsets.dqo_version, sizeof(dispatch_queue_offsets)/sizeof(uint16_t)))
-        {
-            if (process->ReadMemory (thread_dispatch_qaddr, &memory_buffer, data.GetAddressByteSize(), error) == data.GetAddressByteSize())
-            {
-                data_offset = 0;
-                lldb::addr_t queue_addr = data.GetAddress(&data_offset);
-                if (dispatch_queue_offsets.dqo_version >= 4)
-                {
-                    // libdispatch versions 4+, pointer to dispatch name is in the 
-                    // queue structure.
-                    lldb::addr_t pointer_to_label_address = queue_addr + dispatch_queue_offsets.dqo_label;
-                    if (process->ReadMemory (pointer_to_label_address, &memory_buffer, data.GetAddressByteSize(), error) == data.GetAddressByteSize())
-                    {
-                        data_offset = 0;
-                        lldb::addr_t label_addr = data.GetAddress(&data_offset);
-                        process->ReadCStringFromMemory (label_addr, dispatch_queue_name, error);
-                    }
-                }
-                else
-                {
-                    // libdispatch versions 1-3, dispatch name is a fixed width char array
-                    // in the queue structure.
-                    lldb::addr_t label_addr = queue_addr + dispatch_queue_offsets.dqo_label;
-                    dispatch_queue_name.resize(dispatch_queue_offsets.dqo_label_size, '\0');
-                    size_t bytes_read = process->ReadMemory (label_addr, &dispatch_queue_name[0], dispatch_queue_offsets.dqo_label_size, error);
-                    if (bytes_read < dispatch_queue_offsets.dqo_label_size)
-                        dispatch_queue_name.erase (bytes_read);
-                }
-            }
-        }
-    }
-    return dispatch_queue_name;
-}
-
-lldb::queue_id_t
-PlatformDarwin::GetQueueIDForThreadQAddress (Process *process, lldb::addr_t dispatch_qaddr)
-{
-    if (dispatch_qaddr == LLDB_INVALID_ADDRESS || dispatch_qaddr == 0 || process == NULL)
-        return LLDB_INVALID_QUEUE_ID;
-
-    Error error;
-    uint32_t ptr_size = process->GetTarget().GetArchitecture().GetAddressByteSize();
-    uint64_t this_thread_queue_id = process->ReadUnsignedIntegerFromMemory (dispatch_qaddr, ptr_size, LLDB_INVALID_QUEUE_ID, error);
-    if (!error.Success())
-        return LLDB_INVALID_QUEUE_ID;
-
-    return this_thread_queue_id;
-}
-
-
 bool
 PlatformDarwin::x86GetSupportedArchitectureAtIndex (uint32_t idx, ArchSpec &arch)
 {
-    if (idx == 0)
+    ArchSpec host_arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture);
+    if (host_arch.GetCore() == ArchSpec::eCore_x86_64_x86_64h)
     {
-        arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture);
-        return arch.IsValid();
-    }
-    else if (idx == 1)
-    {
-        ArchSpec platform_arch (Host::GetArchitecture (Host::eSystemDefaultArchitecture));
-        ArchSpec platform_arch64 (Host::GetArchitecture (Host::eSystemDefaultArchitecture64));
-        if (platform_arch.IsExactMatch(platform_arch64))
+        switch (idx)
         {
-            // This macosx platform supports both 32 and 64 bit. Since we already
-            // returned the 64 bit arch for idx == 0, return the 32 bit arch 
-            // for idx == 1
-            arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture32);
+            case 0:
+                arch = host_arch;
+                return true;
+
+            case 1:
+                arch.SetTriple("x86_64-apple-macosx");
+                return true;
+
+            case 2:
+                arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture32);
+                return true;
+
+            default: return false;
+        }
+    }
+    else
+    {
+        if (idx == 0)
+        {
+            arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture);
             return arch.IsValid();
         }
+        else if (idx == 1)
+        {
+            ArchSpec platform_arch (Host::GetArchitecture (Host::eSystemDefaultArchitecture));
+            ArchSpec platform_arch64 (Host::GetArchitecture (Host::eSystemDefaultArchitecture64));
+            if (platform_arch.IsExactMatch(platform_arch64))
+            {
+                // This macosx platform supports both 32 and 64 bit. Since we already
+                // returned the 64 bit arch for idx == 0, return the 32 bit arch 
+                // for idx == 1
+                arch = Host::GetArchitecture (Host::eSystemDefaultArchitecture32);
+                return arch.IsValid();
+            }
+        }
     }
     return false;
 }
@@ -987,20 +951,51 @@ bool
 PlatformDarwin::ARMGetSupportedArchitectureAtIndex (uint32_t idx, ArchSpec &arch)
 {
     ArchSpec system_arch (GetSystemArchitecture());
+
     const ArchSpec::Core system_core = system_arch.GetCore();
     switch (system_core)
     {
     default:
         switch (idx)
         {
-            case  0: arch.SetTriple ("armv7-apple-ios");    return true;
-            case  1: arch.SetTriple ("armv7f-apple-ios");   return true;
-            case  2: arch.SetTriple ("armv7k-apple-ios");   return true;
-            case  3: arch.SetTriple ("armv7s-apple-ios");   return true;
-            case  4: arch.SetTriple ("armv7m-apple-ios");   return true;
-            case  5: arch.SetTriple ("armv7em-apple-ios");  return true;
-            case  6: arch.SetTriple ("armv6-apple-ios");    return true;
+            case  0: arch.SetTriple ("arm64-apple-ios");    return true;
+            case  1: arch.SetTriple ("armv7-apple-ios");    return true;
+            case  2: arch.SetTriple ("armv7f-apple-ios");   return true;
+            case  3: arch.SetTriple ("armv7k-apple-ios");   return true;
+            case  4: arch.SetTriple ("armv7s-apple-ios");   return true;
+            case  5: arch.SetTriple ("armv7m-apple-ios");   return true;
+            case  6: arch.SetTriple ("armv7em-apple-ios");  return true;
             case  7: arch.SetTriple ("armv6m-apple-ios");   return true;
+            case  8: arch.SetTriple ("armv6-apple-ios");    return true;
+            case  9: arch.SetTriple ("armv5-apple-ios");    return true;
+            case 10: arch.SetTriple ("armv4-apple-ios");    return true;
+            case 11: arch.SetTriple ("arm-apple-ios");      return true;
+            case 12: arch.SetTriple ("thumbv7-apple-ios");  return true;
+            case 13: arch.SetTriple ("thumbv7f-apple-ios"); return true;
+            case 14: arch.SetTriple ("thumbv7k-apple-ios"); return true;
+            case 15: arch.SetTriple ("thumbv7s-apple-ios"); return true;
+            case 16: arch.SetTriple ("thumbv7m-apple-ios"); return true;
+            case 17: arch.SetTriple ("thumbv7em-apple-ios"); return true;
+            case 18: arch.SetTriple ("thumbv6m-apple-ios"); return true;
+            case 19: arch.SetTriple ("thumbv6-apple-ios");  return true;
+            case 20: arch.SetTriple ("thumbv5-apple-ios");  return true;
+            case 21: arch.SetTriple ("thumbv4t-apple-ios"); return true;
+            case 22: arch.SetTriple ("thumb-apple-ios");    return true;
+            default: break;
+        }
+        break;
+
+    case ArchSpec::eCore_arm_arm64:
+        switch (idx)
+        {
+            case  0: arch.SetTriple ("arm64-apple-ios");   return true;
+            case  1: arch.SetTriple ("armv7s-apple-ios");   return true;
+            case  2: arch.SetTriple ("armv7f-apple-ios");   return true;
+            case  3: arch.SetTriple ("armv7m-apple-ios");   return true;
+            case  4: arch.SetTriple ("armv7em-apple-ios");  return true;
+            case  5: arch.SetTriple ("armv7-apple-ios");    return true;
+            case  6: arch.SetTriple ("armv6m-apple-ios");   return true;
+            case  7: arch.SetTriple ("armv6-apple-ios");    return true;
             case  8: arch.SetTriple ("armv5-apple-ios");    return true;
             case  9: arch.SetTriple ("armv4-apple-ios");    return true;
             case 10: arch.SetTriple ("arm-apple-ios");      return true;
@@ -1010,8 +1005,8 @@ PlatformDarwin::ARMGetSupportedArchitectureAtIndex (uint32_t idx, ArchSpec &arch
             case 14: arch.SetTriple ("thumbv7s-apple-ios"); return true;
             case 15: arch.SetTriple ("thumbv7m-apple-ios"); return true;
             case 16: arch.SetTriple ("thumbv7em-apple-ios"); return true;
-            case 17: arch.SetTriple ("thumbv6-apple-ios");  return true;
-            case 18: arch.SetTriple ("thumbv6m-apple-ios"); return true;
+            case 17: arch.SetTriple ("thumbv6m-apple-ios"); return true;
+            case 18: arch.SetTriple ("thumbv6-apple-ios");  return true;
             case 19: arch.SetTriple ("thumbv5-apple-ios");  return true;
             case 20: arch.SetTriple ("thumbv4t-apple-ios"); return true;
             case 21: arch.SetTriple ("thumb-apple-ios");    return true;
@@ -1333,7 +1328,7 @@ PlatformDarwin::SetThreadCreationBreakpoint (Target &target)
     };
 
     FileSpecList bp_modules;
-    for (size_t i = 0; i < sizeof(g_bp_modules)/sizeof(const char *); i++)
+    for (size_t i = 0; i < llvm::array_lengthof(g_bp_modules); i++)
     {
         const char *bp_module = g_bp_modules[i];
         bp_modules.Append(FileSpec(bp_module, false));
@@ -1345,7 +1340,7 @@ PlatformDarwin::SetThreadCreationBreakpoint (Target &target)
     bp_sp = target.CreateBreakpoint (&bp_modules,
                                      NULL,
                                      g_bp_names,
-                                     sizeof(g_bp_names)/sizeof(const char *),
+                                     llvm::array_lengthof(g_bp_names),
                                      eFunctionNameTypeFull,
                                      skip_prologue,
                                      internal,
@@ -1406,3 +1401,9 @@ PlatformDarwin::GetResumeCountForLaunchInfo (ProcessLaunchInfo &launch_info)
     else
         return 1;
 }
+
+void
+PlatformDarwin::CalculateTrapHandlerSymbolNames ()
+{   
+    m_trap_handlers.push_back (ConstString ("_sigtramp"));
+}   
