@@ -94,7 +94,7 @@ PlatformDarwinKernel::CreateInstance (bool force, const ArchSpec *arch)
                 break;
                 
             // Only accept "unknown" for vendor if the host is Apple and
-            // it "unknown" wasn't specified (it was just returned becasue it
+            // it "unknown" wasn't specified (it was just returned because it
             // was NOT specified)
             case llvm::Triple::UnknownArch:
                 create = !arch->TripleVendorWasSpecified();
@@ -111,7 +111,7 @@ PlatformDarwinKernel::CreateInstance (bool force, const ArchSpec *arch)
                 case llvm::Triple::MacOSX:
                     break;
                 // Only accept "vendor" for vendor if the host is Apple and
-                // it "unknown" wasn't specified (it was just returned becasue it
+                // it "unknown" wasn't specified (it was just returned because it
                 // was NOT specified)
                 case llvm::Triple::UnknownOS:
                     create = !arch->TripleOSWasSpecified();
@@ -133,6 +133,7 @@ PlatformDarwinKernel::CreateInstance (bool force, const ArchSpec *arch)
             is_ios_debug_session = eLazyBoolNo;
             break;
         case llvm::Triple::arm:
+        case llvm::Triple::arm64:
         case llvm::Triple::thumb:
             is_ios_debug_session = eLazyBoolYes;
             break;
@@ -649,13 +650,23 @@ PlatformDarwinKernel::ExamineKextForMatchingUUID (const FileSpec &kext_bundle_pa
 bool
 PlatformDarwinKernel::GetSupportedArchitectureAtIndex (uint32_t idx, ArchSpec &arch)
 {
-#if defined (__arm__)
+#if defined (__arm__) || defined (__arm64__) || defined (__aarch64__)
     return ARMGetSupportedArchitectureAtIndex (idx, arch);
 #else
     return x86GetSupportedArchitectureAtIndex (idx, arch);
 #endif
 }
 
+void
+PlatformDarwinKernel::CalculateTrapHandlerSymbolNames ()
+{   
+    m_trap_handlers.push_back(ConstString ("trap_from_kernel"));
+    m_trap_handlers.push_back(ConstString ("hndl_double_fault"));
+    m_trap_handlers.push_back(ConstString ("hndl_allintrs"));
+    m_trap_handlers.push_back(ConstString ("hndl_alltraps"));
+    m_trap_handlers.push_back(ConstString ("interrupt"));
+}
+
 #else  // __APPLE__
 
 // Since DynamicLoaderDarwinKernel is compiled in for all systems, and relies on
@@ -673,5 +684,4 @@ PlatformDarwinKernel::GetPluginNameStatic ()
     return g_name;
 }
 
-
 #endif // __APPLE__
