@@ -297,7 +297,41 @@ ProcessKDP::DoConnectRemote (Stream *strm, const char *remote_url)
 
                     if (m_comm.RemoteIsEFI ())
                     {
-                        m_dyld_plugin_name = DynamicLoaderStatic::GetPluginNameStatic();
+                        // Select an invalid plugin name for the dynamic loader so one doesn't get used
+                        // since EFI does its own manual loading via python scripting
+                        static ConstString g_none_dynamic_loader("none");
+                        m_dyld_plugin_name = g_none_dynamic_loader;
+
+                        if (kernel_uuid.IsValid()) {
+                            // If EFI passed in a UUID= try to lookup UUID
+                            // The slide will not be provided. But the UUID
+                            // lookup will be used to launch EFI debug scripts
+                            // from the dSYM, that can load all of the symbols.
+                            ModuleSpec module_spec;
+                            module_spec.GetUUID() = kernel_uuid;
+                            module_spec.GetArchitecture() = m_target.GetArchitecture();
+
+                            // Lookup UUID locally, before attempting dsymForUUID like action
+                            module_spec.GetSymbolFileSpec() = Symbols::LocateExecutableSymbolFile(module_spec);
+                            if (module_spec.GetSymbolFileSpec())
+                                 module_spec.GetFileSpec() = Symbols::LocateExecutableObjectFile (module_spec);
+                            if (!module_spec.GetSymbolFileSpec() || !module_spec.GetSymbolFileSpec())
+                                 Symbols::DownloadObjectAndSymbolFile (module_spec, true);
+
+                            if (module_spec.GetFileSpec().Exists())
+                            {
+                                ModuleSP module_sp(new Module (module_spec.GetFileSpec(), m_target.GetArchitecture()));
+                                if (module_sp.get() && module_sp->MatchesModuleSpec (module_spec))
+                                {
+                                    // Get the current target executable
+                                    ModuleSP exe_module_sp (m_target.GetExecutableModule ());
+
+                                    // Make sure you don't already have the right module loaded and they will be uniqued
+                                    if (exe_module_sp.get() != module_sp.get())
+                                        m_target.SetExecutableModule (module_sp, false);
+                                }
+                            }
+                        }
                     }
                     else if (m_comm.RemoteIsDarwinKernel ())
                     {
@@ -360,7 +394,7 @@ ProcessKDP::DoConnectRemote (Stream *strm, const char *remote_url)
 //----------------------------------------------------------------------
 Error
 ProcessKDP::DoLaunch (Module *exe_module, 
-                      const ProcessLaunchInfo &launch_info)
+                      ProcessLaunchInfo &launch_info)
 {
     Error error;
     error.SetErrorString ("launching not supported in kdp-remote plug-in");
@@ -385,7 +419,7 @@ ProcessKDP::DoAttachToProcessWithID (lldb::pid_t attach_pid, const ProcessAttach
 }
 
 Error
-ProcessKDP::DoAttachToProcessWithName (const char *process_name, bool wait_for_launch, const ProcessAttachInfo &attach_info)
+ProcessKDP::DoAttachToProcessWithName (const char *process_name, const ProcessAttachInfo &attach_info)
 {
     Error error;
     error.SetErrorString ("attach to process by name is not suppported in kdp remote debugging");
@@ -599,10 +633,6 @@ ProcessKDP::DoDetach(bool keep_stopped)
     }
     else
     {
-        DisableAllBreakpointSites ();
-        
-        m_thread_list.DiscardThreadPlans();
-        
         // If we are going to keep the target stopped, then don't send the disconnect message.
         if (!keep_stopped && m_comm.IsConnected())
         {
@@ -651,8 +681,32 @@ ProcessKDP::IsAlive ()
 size_t
 ProcessKDP::DoReadMemory (addr_t addr, void *buf, size_t size, Error &error)
 {
+    uint8_t *data_buffer = (uint8_t *) buf;
     if (m_comm.IsConnected())
-        return m_comm.SendRequestReadMemory (addr, buf, size, error);
+    {
+        const size_t max_read_size = 512;
+        size_t total_bytes_read = 0;
+
+        // Read the requested amount of memory in 512 byte chunks
+        while (total_bytes_read < size)
+        {
+            size_t bytes_to_read_this_request = size - total_bytes_read;
+            if (bytes_to_read_this_request > max_read_size)
+            {
+                bytes_to_read_this_request = max_read_size;
+            }
+            size_t bytes_read = m_comm.SendRequestReadMemory (addr + total_bytes_read, 
+                                                              data_buffer + total_bytes_read, 
+                                                              bytes_to_read_this_request, error);
+            total_bytes_read += bytes_read;
+            if (error.Fail() || bytes_read == 0)
+            {
+                return total_bytes_read;
+            }
+        }
+
+        return total_bytes_read;
+    }
     error.SetErrorString ("not connected");
     return 0;
 }
