@@ -21,9 +21,9 @@
 #include "lldb/Core/Broadcaster.h"
 #include "lldb/Core/ConstString.h"
 #include "lldb/Core/Error.h"
-#include "lldb/Core/InputReader.h"
 #include "lldb/Core/StreamString.h"
 #include "lldb/Core/StringList.h"
+#include "lldb/Core/StructuredData.h"
 #include "lldb/Core/ThreadSafeValue.h"
 #include "lldb/Target/Process.h"
 #include "lldb/Target/Thread.h"
@@ -86,11 +86,14 @@ public:
 
     virtual lldb_private::Error
     DoLaunch (lldb_private::Module *exe_module, 
-              const lldb_private::ProcessLaunchInfo &launch_info);
+              lldb_private::ProcessLaunchInfo &launch_info);
 
     virtual void
     DidLaunch ();
 
+    lldb_private::UnixSignals&
+    GetUnixSignals () override;
+
     virtual lldb_private::Error
     WillAttachToProcessWithID (lldb::pid_t pid);
 
@@ -111,7 +114,6 @@ public:
     
     virtual lldb_private::Error
     DoAttachToProcessWithName (const char *process_name,
-                               bool wait_for_launch,
                                const lldb_private::ProcessAttachInfo &attach_info);
 
     virtual void
@@ -222,6 +224,18 @@ public:
     {
         return m_gdb_comm;
     }
+    
+    virtual lldb_private::Error
+    SendEventData(const char *data);
+
+    //----------------------------------------------------------------------
+    // Override SetExitStatus so we can disconnect from the remote GDB server
+    //----------------------------------------------------------------------
+    virtual bool
+    SetExitStatus (int exit_status, const char *cstr);
+
+    void
+    SetUserSpecifiedMaxMemoryTransferSize (uint64_t user_specified_max);
 
 protected:
     friend class ThreadGDBRemote;
@@ -277,10 +291,7 @@ protected:
                       lldb_private::ThreadList &new_thread_list);
 
     lldb_private::Error
-    StartDebugserverProcess (const char *debugserver_url);
-    
-    lldb_private::Error
-    StartDebugserverProcess (const char *debugserver_url, const lldb_private::ProcessInfo &process_info);
+    LaunchAndConnectToDebugserver (const lldb_private::ProcessInfo &process_info);
 
     void
     KillDebugserverProcess ();
@@ -297,6 +308,15 @@ protected:
     bool
     ParseRegisters(lldb_private::ScriptInterpreterObject *registers_array);
 
+    const lldb::DataBufferSP
+    GetAuxvData() override;
+
+    lldb_private::StructuredData::ObjectSP
+    GetExtendedInfoForThread (lldb::tid_t tid);
+
+    void
+    GetMaxMemorySize();
+
     //------------------------------------------------------------------
     /// Broadcaster event bits definitions.
     //------------------------------------------------------------------
@@ -332,14 +352,17 @@ protected:
     tid_sig_collection m_continue_C_tids; // 'C' for continue with signal
     tid_collection m_continue_s_tids;                  // 's' for step
     tid_sig_collection m_continue_S_tids; // 'S' for step with signal
-    size_t m_max_memory_size;       // The maximum number of bytes to read/write when reading and writing memory
+    uint64_t m_max_memory_size;       // The maximum number of bytes to read/write when reading and writing memory
+    uint64_t m_remote_stub_max_memory_size;    // The maximum memory size the remote gdb stub can handle
     MMapMap m_addr_to_mmap_size;
     lldb::BreakpointSP m_thread_create_bp_sp;
     bool m_waiting_for_attach;
     bool m_destroy_tried_resuming;
     lldb::CommandObjectSP m_command_sp;
     int64_t m_breakpoint_pc_offset;
-    
+    std::shared_ptr<lldb_private::UnixSignals> m_unix_signals_sp;
+
+
     bool
     StartAsyncThread ();
 
@@ -375,13 +398,6 @@ protected:
     GetDispatchQueueNameForThread (lldb::addr_t thread_dispatch_qaddr,
                                    std::string &dispatch_queue_name);
 
-    static size_t
-    AttachInputReaderCallback (void *baton, 
-                               lldb_private::InputReader *reader, 
-                               lldb::InputReaderAction notification,
-                               const char *bytes, 
-                               size_t bytes_len);
-
     lldb_private::DynamicLoader *
     GetDynamicLoader ();
 
