@@ -11,7 +11,7 @@ from lldbtest import *
 
 class CommandSourceTestCase(TestBase):
 
-    mydir = os.path.join("functionalities", "command_source")
+    mydir = TestBase.compute_mydir(__file__)
 
     def test_command_source(self):
         """Test that lldb command "command source" works correctly."""
@@ -20,21 +20,14 @@ class CommandSourceTestCase(TestBase):
         # the "my" package that defines the date() function.
         self.runCmd("command source .lldb")
 
-        # Let's temporarily redirect the stdout to our StringIO session object
-        # in order to capture the script evaluation output.
-        old_stdout = sys.stdout
-        session = StringIO.StringIO()
-        sys.stdout = session
-
         # Python should evaluate "my.date()" successfully.
-        # Pass 'check=False' so that sys.stdout gets restored unconditionally.
-        self.runCmd("script my.date()", check=False)
-
-        # Now restore stdout to the way we were. :-)
-        sys.stdout = old_stdout
+        command_interpreter = self.dbg.GetCommandInterpreter()
+        self.assertTrue(command_interpreter, VALID_COMMAND_INTERPRETER)
+        result = lldb.SBCommandReturnObject()
+        command_interpreter.HandleCommand("script my.date()", result)
 
         import datetime
-        self.expect(session.getvalue(), "script my.date() runs successfully",
+        self.expect(result.GetOutput(), "script my.date() runs successfully",
                     exe=False,
             substrs = [str(datetime.date.today())])
 
