@@ -5,16 +5,16 @@ Test lldb 'process connect' command.
 import os
 import unittest2
 import lldb
-import pexpect
 from lldbtest import *
 
 class ConnectRemoteTestCase(TestBase):
 
-    mydir = os.path.join("functionalities", "connect_remote")
+    mydir = TestBase.compute_mydir(__file__)
 
     def test_connect_remote(self):
         """Test "process connect connect:://localhost:12345"."""
 
+        import pexpect
         # First, we'll start a fake debugserver (a simple echo server).
         fakeserver = pexpect.spawn('./EchoServer.py')
 
@@ -31,10 +31,7 @@ class ConnectRemoteTestCase(TestBase):
         fakeserver.expect_exact('Listening on localhost:12345')
 
         # Connect to the fake server....
-        if sys.platform.startswith('freebsd') or sys.platform.startswith("linux"):
-            self.runCmd("process connect -p gdb-remote connect://localhost:12345")
-        else:
-            self.runCmd("process connect connect://localhost:12345")
+        self.runCmd("process connect -p gdb-remote connect://localhost:12345")
 
 
 if __name__ == '__main__':
