@@ -25,7 +25,7 @@ signal_names = dict((getattr(signal, n), n) \
 
 class ConcurrentEventsTestCase(TestBase):
 
-    mydir = os.path.join("functionalities", "thread", "concurrent_events")
+    mydir = TestBase.compute_mydir(__file__)
 
     #
     ## Tests for multiple threads that generate a single event.
@@ -37,8 +37,8 @@ class ConcurrentEventsTestCase(TestBase):
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=100)
 
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
     @unittest2.skipIf(TestBase.skipLongRunningTest(), "Skip this long running test")
+    @expectedFailureLinux('llvm.org/pr16714')
     @dwarf_test
     def test_many_watchpoints_dwarf(self):
         """Test 100 watchpoints from 100 threads."""
@@ -63,21 +63,18 @@ class ConcurrentEventsTestCase(TestBase):
     #
     ## Tests for concurrent signal and breakpoint
     #
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_signal_break_dwarf(self):
         """Test signal and a breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=1, num_signal_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_delay_signal_break_dwarf(self):
         """Test (1-second delay) signal and a breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=1, num_delay_signal_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_signal_delay_break_dwarf(self):
         """Test signal and a (1 second delay) breakpoint in multiple threads."""
@@ -89,24 +86,25 @@ class ConcurrentEventsTestCase(TestBase):
     ## Tests for concurrent watchpoint and breakpoint
     #
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_watch_break_dwarf(self):
         """Test watchpoint and a breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=1, num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_delay_watch_break_dwarf(self):
         """Test (1-second delay) watchpoint and a breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=1, num_delay_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
-    def test_watch_break_dwarf(self):
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
+    def test_watch_break_dwarf_delay(self):
         """Test watchpoint and a (1 second delay) breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_delay_breakpoint_threads=1, num_watchpoint_threads=1)
@@ -114,25 +112,25 @@ class ConcurrentEventsTestCase(TestBase):
     #
     ## Tests for concurrent signal and watchpoint
     #
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_signal_watch_dwarf(self):
         """Test a watchpoint and a signal in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_signal_threads=1, num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_delay_signal_watch_dwarf(self):
         """Test a watchpoint and a (1 second delay) signal in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_delay_signal_threads=1, num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_signal_delay_watch_dwarf(self):
         """Test a (1 second delay) watchpoint and a signal in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -142,14 +140,12 @@ class ConcurrentEventsTestCase(TestBase):
     #
     ## Tests for multiple breakpoint threads
     #
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_two_breakpoint_threads_dwarf(self):
         """Test two threads that trigger a breakpoint. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=2)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_breakpoint_one_delay_breakpoint_threads_dwarf(self):
         """Test threads that trigger a breakpoint where one thread has a 1 second delay. """
@@ -157,14 +153,12 @@ class ConcurrentEventsTestCase(TestBase):
         self.do_thread_actions(num_breakpoint_threads=1,
                                num_delay_breakpoint_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_two_breakpoints_one_signal_dwarf(self):
         """Test two threads that trigger a breakpoint and one signal thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=2, num_signal_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_breakpoint_delay_breakpoint_one_signal_dwarf(self):
         """Test two threads that trigger a breakpoint (one with a 1 second delay) and one signal thread. """
@@ -173,24 +167,23 @@ class ConcurrentEventsTestCase(TestBase):
                                num_delay_breakpoint_threads=1,
                                num_signal_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_two_breakpoints_one_delay_signal_dwarf(self):
         """Test two threads that trigger a breakpoint and one (1 second delay) signal thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=2, num_delay_signal_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_two_breakpoints_one_watchpoint_dwarf(self):
         """Test two threads that trigger a breakpoint and one watchpoint thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_breakpoint_threads=2, num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_breakpoints_delayed_breakpoint_one_watchpoint_dwarf(self):
         """Test a breakpoint, a delayed breakpoint, and one watchpoint thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -201,42 +194,42 @@ class ConcurrentEventsTestCase(TestBase):
     #
     ## Tests for multiple watchpoint threads
     #
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_two_watchpoint_threads_dwarf(self):
         """Test two threads that trigger a watchpoint. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_watchpoint_threads=2)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
-    def test_watchpoint_with_delay_waychpoint_threads_dwarf(self):
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
+    def test_watchpoint_with_delay_watchpoint_threads_dwarf(self):
         """Test two threads that trigger a watchpoint where one thread has a 1 second delay. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_watchpoint_threads=1,
                                num_delay_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_two_watchpoints_one_breakpoint_dwarf(self):
         """Test two threads that trigger a watchpoint and one breakpoint thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_watchpoint_threads=2, num_breakpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_two_watchpoints_one_delay_breakpoint_dwarf(self):
         """Test two threads that trigger a watchpoint and one (1 second delay) breakpoint thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_watchpoint_threads=2, num_delay_breakpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_watchpoint_delay_watchpoint_one_breakpoint_dwarf(self):
         """Test two threads that trigger a watchpoint (one with a 1 second delay) and one breakpoint thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -244,9 +237,9 @@ class ConcurrentEventsTestCase(TestBase):
                                num_delay_watchpoint_threads=1,
                                num_breakpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_two_watchpoints_one_signal_dwarf(self):
         """Test two threads that trigger a watchpoint and one signal thread. """
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -256,7 +249,7 @@ class ConcurrentEventsTestCase(TestBase):
     ## Test for watchpoint, signal and breakpoint happening concurrently
     #
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
     def test_signal_watch_break_dwarf(self):
         """Test a signal/watchpoint/breakpoint in multiple threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -265,7 +258,7 @@ class ConcurrentEventsTestCase(TestBase):
                                num_breakpoint_threads=1)
 
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
     def test_signal_watch_break_dwarf(self):
         """Test one signal thread with 5 watchpoint and breakpoint threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -273,9 +266,9 @@ class ConcurrentEventsTestCase(TestBase):
                                num_watchpoint_threads=5,
                                num_breakpoint_threads=5)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
+    @expectedFailureLinux('llvm.org/pr16714')
     def test_signal_watch_break_dwarf(self):
         """Test with 5 watchpoint and breakpoint threads."""
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -286,31 +279,27 @@ class ConcurrentEventsTestCase(TestBase):
     #
     ## Test for crashing threads happening concurrently with other events
     #
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_crash_with_break_dwarf(self):
         """ Test a thread that crashes while another thread hits a breakpoint."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_crash_threads=1, num_breakpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
     def test_crash_with_watchpoint_dwarf(self):
         """ Test a thread that crashes while another thread hits a watchpoint."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_crash_threads=1, num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_crash_with_signal_dwarf(self):
         """ Test a thread that crashes while another thread generates a signal."""
         self.buildDwarf(dictionary=self.getBuildFlags())
         self.do_thread_actions(num_crash_threads=1, num_signal_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
     def test_crash_with_watchpoint_breakpoint_signal_dwarf(self):
         """ Test a thread that crashes while other threads generate a signal and hit a watchpoint and breakpoint. """
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -319,9 +308,8 @@ class ConcurrentEventsTestCase(TestBase):
                                num_signal_threads=1,
                                num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD('llvm.org/pr16706') # Watchpoints fail on FreeBSD
     @dwarf_test
-    @skipIfLinux # llvm.org/pr16714 - LLDB sometimes crashes when setting watchpoints in multithreaded programs
+    @skipIfRemoteDueToDeadlock
     def test_delayed_crash_with_breakpoint_watchpoint_dwarf(self):
         """ Test a thread with a delayed crash while other threads hit a watchpoint and a breakpoint. """
         self.buildDwarf(dictionary=self.getBuildFlags())
@@ -329,7 +317,6 @@ class ConcurrentEventsTestCase(TestBase):
                                num_breakpoint_threads=1,
                                num_watchpoint_threads=1)
 
-    @expectedFailureFreeBSD("llvm.org/pr16696") # threaded inferior not yet implemented on FreeBSD
     @dwarf_test
     def test_delayed_crash_with_breakpoint_signal_dwarf(self):
         """ Test a thread with a delayed crash while other threads generate a signal and hit a breakpoint. """
