@@ -373,6 +373,40 @@ def dwarf_test(func):
     wrapper.__dwarf_test__ = True
     return wrapper
 
+def debugserver_test(func):
+    """Decorate the item as a debugserver test."""
+    if isinstance(func, type) and issubclass(func, unittest2.TestCase):
+        raise Exception("@debugserver_test can only be used to decorate a test method")
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        try:
+            if lldb.dont_do_debugserver_test:
+                self.skipTest("debugserver tests")
+        except AttributeError:
+            pass
+        return func(self, *args, **kwargs)
+
+    # Mark this function as such to separate them from the regular tests.
+    wrapper.__debugserver_test__ = True
+    return wrapper
+
+def llgs_test(func):
+    """Decorate the item as a lldb-gdbserver test."""
+    if isinstance(func, type) and issubclass(func, unittest2.TestCase):
+        raise Exception("@llgs_test can only be used to decorate a test method")
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        try:
+            if lldb.dont_do_llgs_test:
+                self.skipTest("llgs tests")
+        except AttributeError:
+            pass
+        return func(self, *args, **kwargs)
+
+    # Mark this function as such to separate them from the regular tests.
+    wrapper.__llgs_test__ = True
+    return wrapper
+
 def not_remote_testsuite_ready(func):
     """Decorate the item as a test which is not ready yet for remote testsuite."""
     if isinstance(func, type) and issubclass(func, unittest2.TestCase):
@@ -390,294 +424,95 @@ def not_remote_testsuite_ready(func):
     wrapper.__not_ready_for_remote_testsuite_test__ = True
     return wrapper
 
-def expectedFailureGcc(bugnumber=None, compiler_version=["=", None]):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureGcc_easy_wrapper(*args, **kwargs):
+def expectedFailure(expected_fn, bugnumber=None):
+    def expectedFailure_impl(func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
             from unittest2 import case
             self = args[0]
-            test_compiler = self.getCompiler()
             try:
-                bugnumber(*args, **kwargs)
+                func(*args, **kwargs)
             except Exception:
-                if "gcc" in test_compiler and self.expectedCompilerVersion(compiler_version):
-                    raise case._ExpectedFailure(sys.exc_info(),None)
+                if expected_fn(self):
+                    raise case._ExpectedFailure(sys.exc_info(), bugnumber)
                 else:
                     raise
-            if "gcc" in test_compiler:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureGcc_easy_wrapper
-     else:
-        def expectedFailureGcc_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                test_compiler = self.getCompiler()
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "gcc" in test_compiler and self.expectedCompilerVersion(compiler_version):
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "gcc" in test_compiler:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureGcc_impl
+            if expected_fn(self):
+                raise case._UnexpectedSuccess(sys.exc_info(), bugnumber)
+        return wrapper
+    if callable(bugnumber):
+        return expectedFailure_impl(bugnumber)
+    else:
+        return expectedFailure_impl
+
+def expectedFailureCompiler(compiler, compiler_version=None, bugnumber=None):
+    if compiler_version is None:
+        compiler_version=['=', None]
+    def fn(self):
+        return compiler in self.getCompiler() and self.expectedCompilerVersion(compiler_version)
+    return expectedFailure(fn, bugnumber)
 
 def expectedFailureClang(bugnumber=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureClang_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            test_compiler = self.getCompiler()
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "clang" in test_compiler:
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "clang" in test_compiler:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureClang_easy_wrapper
-     else:
-        def expectedFailureClang_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                test_compiler = self.getCompiler()
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "clang" in test_compiler:
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "clang" in test_compiler:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureClang_impl
+    return expectedFailureCompiler('clang', None, bugnumber)
+
+def expectedFailureGcc(bugnumber=None, compiler_version=None):
+    return expectedFailureCompiler('gcc', compiler_version, bugnumber)
 
 def expectedFailureIcc(bugnumber=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureIcc_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            test_compiler = self.getCompiler()
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "icc" in test_compiler:
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "icc" in test_compiler:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureIcc_easy_wrapper
-     else:
-        def expectedFailureIcc_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                test_compiler = self.getCompiler()
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "icc" in test_compiler:
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "icc" in test_compiler:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureIcc_impl
+    return expectedFailureCompiler('icc', None, bugnumber)
 
+def expectedFailureArch(arch, bugnumber=None):
+    def fn(self):
+        return arch in self.getArchitecture()
+    return expectedFailure(fn, bugnumber)
 
 def expectedFailurei386(bugnumber=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailurei386_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            arch = self.getArchitecture()
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "i386" in arch:
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "i386" in arch:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailurei386_easy_wrapper
-     else:
-        def expectedFailurei386_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                arch = self.getArchitecture()
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "i386" in arch:
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "i386" in arch:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailurei386_impl
+    return expectedFailureArch('i386', bugnumber)
 
 def expectedFailurex86_64(bugnumber=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailurex86_64_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            arch = self.getArchitecture()
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "x86_64" in arch:
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "x86_64" in arch:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailurex86_64_easy_wrapper
-     else:
-        def expectedFailurex86_64_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                arch = self.getArchitecture()
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "x86_64" in arch:
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "x86_64" in arch:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailurex86_64_impl
+    return expectedFailureArch('x86_64', bugnumber)
+
+def expectedFailureOS(os, bugnumber=None, compilers=None):
+    def fn(self):
+        return os in sys.platform and self.expectedCompiler(compilers)
+    return expectedFailure(fn, bugnumber)
+
+def expectedFailureDarwin(bugnumber=None, compilers=None):
+    return expectedFailureOS('darwin', bugnumber, compilers)
 
 def expectedFailureFreeBSD(bugnumber=None, compilers=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureFreeBSD_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            platform = sys.platform
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "freebsd" in platform and self.expectedCompiler(compilers):
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "freebsd" in platform and self.expectedCompiler(compilers):
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureFreeBSD_easy_wrapper
-     else:
-        def expectedFailureFreeBSD_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                platform = sys.platform
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "freebsd" in platform and self.expectedCompiler(compilers):
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "freebsd" in platform and self.expectedCompiler(compilers):
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureFreeBSD_impl
+    return expectedFailureOS('freebsd', bugnumber, compilers)
 
 def expectedFailureLinux(bugnumber=None, compilers=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureLinux_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
-            self = args[0]
-            platform = sys.platform
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "linux" in platform and self.expectedCompiler(compilers):
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "linux" in platform and self.expectedCompiler(compilers):
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureLinux_easy_wrapper
-     else:
-        def expectedFailureLinux_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                platform = sys.platform
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "linux" in platform and self.expectedCompiler(compilers):
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "linux" in platform and self.expectedCompiler(compilers):
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureLinux_impl
+    return expectedFailureOS('linux', bugnumber, compilers)
 
-def expectedFailureDarwin(bugnumber=None):
-     if callable(bugnumber):
-        @wraps(bugnumber)
-        def expectedFailureDarwin_easy_wrapper(*args, **kwargs):
-            from unittest2 import case
+def skipIfRemote(func):
+    """Decorate the item to skip tests if testing remotely."""
+    if isinstance(func, type) and issubclass(func, unittest2.TestCase):
+        raise Exception("@skipIfRemote can only be used to decorate a test method")
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        from unittest2 import case
+        if lldb.remote_platform:
             self = args[0]
-            platform = sys.platform
-            try:
-                bugnumber(*args, **kwargs)
-            except Exception:
-                if "darwin" in platform:
-                    raise case._ExpectedFailure(sys.exc_info(),None)
-                else:
-                    raise
-            if "darwin" in platform:
-                raise case._UnexpectedSuccess(sys.exc_info(),None)
-        return expectedFailureDarwin_easy_wrapper
-     else:
-        def expectedFailureDarwin_impl(func):
-              @wraps(func)
-              def wrapper(*args, **kwargs):
-                from unittest2 import case
-                self = args[0]
-                platform = sys.platform
-                try:
-                    func(*args, **kwargs)
-                except Exception:
-                    if "darwin" in platform:
-                        raise case._ExpectedFailure(sys.exc_info(),bugnumber)
-                    else:
-                        raise
-                if "darwin" in platform:
-                    raise case._UnexpectedSuccess(sys.exc_info(),bugnumber)
-              return wrapper
-        return expectedFailureDarwin_impl
+            self.skipTest("skip on remote platform")
+        else:
+            func(*args, **kwargs)
+    return wrapper
+
+def skipIfRemoteDueToDeadlock(func):
+    """Decorate the item to skip tests if testing remotely due to the test deadlocking."""
+    if isinstance(func, type) and issubclass(func, unittest2.TestCase):
+        raise Exception("@skipIfRemote can only be used to decorate a test method")
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        from unittest2 import case
+        if lldb.remote_platform:
+            self = args[0]
+            self.skipTest("skip on remote platform (deadlocks)")
+        else:
+            func(*args, **kwargs)
+    return wrapper
 
 def skipIfFreeBSD(func):
     """Decorate the item to skip tests that should be skipped on FreeBSD."""
@@ -801,11 +636,19 @@ class Base(unittest2.TestCase):
 
     # Keep track of the old current working directory.
     oldcwd = None
-
+    
+    @staticmethod
+    def compute_mydir(test_file):
+        '''Subclasses should call this function to correctly calculate the required "mydir" attribute as follows: 
+            
+            mydir = TestBase.compute_mydir(__file__)'''
+        test_dir = os.path.dirname(test_file)
+        return test_dir[len(os.environ["LLDB_TEST"])+1:]
+    
     def TraceOn(self):
         """Returns True if we are in trace mode (tracing detailed test execution)."""
         return traceAlways
-
+    
     @classmethod
     def setUpClass(cls):
         """
@@ -1365,6 +1208,14 @@ class Base(unittest2.TestCase):
     # Build methods supported through a plugin interface
     # ==================================================
 
+    def getstdlibFlag(self):
+        """ Returns the proper -stdlib flag, or empty if not required."""
+        if sys.platform.startswith("darwin") or sys.platform.startswith("freebsd"):
+            stdlibflag = "-stdlib=libc++"
+        else:
+            stdlibflag = ""
+        return stdlibflag
+
     def getstdFlag(self):
         """ Returns the proper stdflag. """
         if "gcc" in self.getCompiler() and "4.6" in self.getCompilerVersion():
@@ -1379,19 +1230,20 @@ class Base(unittest2.TestCase):
         """
 
         stdflag = self.getstdFlag()
+        stdlibflag = self.getstdlibFlag()
 
         if sys.platform.startswith("darwin"):
             dsym = os.path.join(self.lib_dir, 'LLDB.framework', 'LLDB')
             d = {'CXX_SOURCES' : sources,
                  'EXE' : exe_name,
-                 'CFLAGS_EXTRAS' : "%s -stdlib=libc++" % stdflag,
+                 'CFLAGS_EXTRAS' : "%s %s" % (stdflag, stdlibflag),
                  'FRAMEWORK_INCLUDES' : "-F%s" % self.lib_dir,
                  'LD_EXTRAS' : "%s -Wl,-rpath,%s" % (dsym, self.lib_dir),
                 }
         elif sys.platform.startswith('freebsd') or sys.platform.startswith("linux") or os.environ.get('LLDB_BUILD_TYPE') == 'Makefile':
             d = {'CXX_SOURCES' : sources, 
                  'EXE' : exe_name,
-                 'CFLAGS_EXTRAS' : "%s -I%s" % (stdflag, os.path.join(os.environ["LLDB_SRC"], "include")),
+                 'CFLAGS_EXTRAS' : "%s %s -I%s" % (stdflag, stdlibflag, os.path.join(os.environ["LLDB_SRC"], "include")),
                  'LD_EXTRAS' : "-L%s -llldb" % self.lib_dir}
         if self.TraceOn():
             print "Building LLDB Driver (%s) from sources %s" % (exe_name, sources)
@@ -1475,7 +1327,7 @@ class Base(unittest2.TestCase):
                 cflags += "c++0x"
             else:
                 cflags += "c++11"
-        if sys.platform.startswith("darwin"):
+        if sys.platform.startswith("darwin") or sys.platform.startswith("freebsd"):
             cflags += " -stdlib=libc++"
         elif "clang" in self.getCompiler():
             cflags += " -stdlib=libstdc++"
@@ -1656,7 +1508,7 @@ class TestBase(Base):
                 self.runCmd("file %s" % arg)
                 target = self.dbg.GetSelectedTarget()
                 #
-                # SBTarget.LaunchSimple() currently not working for remote platform?
+                # SBtarget.LaunchSimple () currently not working for remote platform?
                 # johnny @ 04/23/2012
                 #
                 def DecoratedLaunchSimple(argv, envp, wd):
@@ -1684,6 +1536,18 @@ class TestBase(Base):
         if lldb.pre_flight:
             lldb.pre_flight(self)
 
+        if lldb.remote_platform:
+            #remote_test_dir = os.path.join(lldb.remote_platform_working_dir, self.mydir)
+            remote_test_dir = os.path.join(lldb.remote_platform_working_dir, 
+                                           self.getArchitecture(), 
+                                           str(self.test_number), 
+                                           self.mydir)
+            error = lldb.remote_platform.MakeDirectory(remote_test_dir, 0700)
+            if error.Success():
+                lldb.remote_platform.SetWorkingDirectory(remote_test_dir)
+            else:
+                print "error: making remote directory '%s': %s" % (remote_test_dir, error)
+    
     # utility methods that tests can use to access the current objects
     def target(self):
         if not self.dbg:
@@ -1705,6 +1569,15 @@ class TestBase(Base):
             raise Exception('Invalid debugger instance')
         return self.dbg.GetSelectedTarget().GetProcess().GetSelectedThread().GetSelectedFrame()
 
+    def get_process_working_directory(self):
+        '''Get the working directory that should be used when launching processes for local or remote processes.'''
+        if lldb.remote_platform:
+            # Remote tests set the platform working directory up in TestBase.setUp()
+            return lldb.remote_platform.GetWorkingDirectory()
+        else:
+            # local tests change directory into each test subdirectory
+            return os.getcwd() 
+    
     def tearDown(self):
         #import traceback
         #traceback.print_stack()
